"""
run.py is an entry to lintest automation framework.
please copy this module to your project.
the structure of your project should be like this:
your_project_folder
|----  settings/          # settings will be auto-generated by lintest if doesn't exists
|----  common/            # the common libraries
|----  tests/             # all the testcases must be under "tests" package
|----  run.py             # the entry of lintest

@author: Wang Lin
"""

import os
import sys
import traceback


os.environ["LIN_TEST_PROJECT_PATH"] = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))

try:
    os.system(
        "pip3 install --upgrade --extra-index-url http://47.117.134.28/simple/ lintest --trusted-host 47.117.134.28")
    from lintest import main
except ImportError:
    traceback.print_exc()

main.run_with(sys.argv)
